package com.link.core.basic.aop;

import com.link.core.basic.service.BasicServiceException;
import com.link.core.basic.service.ServiceException;
import com.link.core.util.StringUtils;
import java.util.HashMap;
import java.util.Map;
import org.apache.ibatis.exceptions.PersistenceException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.mybatis.spring.MyBatisSystemException;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.jdbc.BadSqlGrammarException;

@Aspect
public class ExceptionAspect {
   private static final Logger logger = LogManager.getLogger(ExceptionAspect.class);
   private static final String TURE = "true";
   private static final String WxPayException = "com.wechat.pay.java.core.exception.ServiceException";

   @Pointcut("(@within(org.springframework.stereotype.Service) && execution(public * *(..))) || within(com.link.core.basic.service.BasicServiceImpl)")
   private void servicePointcut() {
   }

   @AfterThrowing(
      pointcut = "servicePointcut()",
      throwing = "e"
   )
   public void handle(JoinPoint point, Exception e) {
      String signature = point.getSignature().toString();
      System.err.println(signature);
      if (!(e instanceof ServiceException)) {
         String errorClass = e.getClass().getName();
         if (!"com.wechat.pay.java.core.exception.ServiceException".equals(errorClass)) {
            Map<String, String> exceptionMsg = this.getMessage(e);
            String errorMsg = (String)exceptionMsg.get("message");
            logger.error("ExceptionAspect捕获异常：", e);
            if ("true".equals(exceptionMsg.get("showDetailErrMsgFlag"))) {
               throw new BasicServiceException(errorMsg, e.getMessage(), (String)exceptionMsg.get("code"));
            } else {
               throw new BasicServiceException(errorMsg, "", (String)exceptionMsg.get("code"));
            }
         }
      }
   }

   private Map<String, String> getMessage(Exception e) {
      Map<String, String> result = new HashMap(4);
      String message = e.getMessage();
      String showDetailErrMsgFlag = "false";
      String code = "400";
      if (e instanceof NullPointerException) {
         message = "服务异常（null）";
         showDetailErrMsgFlag = "true";
         code = "404";
      } else if (e instanceof BadSqlGrammarException) {
         message = "语法错误";
         showDetailErrMsgFlag = "false";
         code = "403";
      } else if (e instanceof DuplicateKeyException) {
         message = "唯一性冲突";
         showDetailErrMsgFlag = "false";
         code = "402";
      } else if (e instanceof PersistenceException) {
         message = "MyBatis未找到对应方法，请检查字段或方法名是否存在等";
         logger.error("Mybatis未找到对应方法异常：" + e.getMessage());
         showDetailErrMsgFlag = "false";
         code = "406";
      } else if (e instanceof DataIntegrityViolationException) {
         message = "新建或更新的数据长度超出范围或违反完整性约束";
         showDetailErrMsgFlag = "false";
         code = "407";
      } else if (e instanceof MyBatisSystemException) {
         message = this.getBasicServiceExceptionMsg(e);
         if (StringUtils.isBlank(message)) {
            message = "MyBatis异常，请检查相关语句";
         }

         showDetailErrMsgFlag = "false";
         code = "408";
      }

      result.put("message", message);
      result.put("showDetailErrMsgFlag", showDetailErrMsgFlag);
      result.put("code", code);
      return result;
   }

   private String getBasicServiceExceptionMsg(Exception e) {
      String msg = null;

      try {
         if (e instanceof BasicServiceException) {
            msg = e.getMessage();
            return msg;
         }

         int maxLoopTime = 10;
         Throwable exceptionTemp = e.getCause();

         for(int i = 0; i < maxLoopTime; ++i) {
            if (exceptionTemp instanceof BasicServiceException) {
               msg = exceptionTemp.getMessage();
               break;
            }

            exceptionTemp = exceptionTemp.getCause();
         }
      } catch (Throwable var6) {
         Throwable exception = var6;
         exception.printStackTrace();
      }

      return msg;
   }
}
